function [responses, decisionLevel, responseValues, nHypos]= getHyperClassifierResponse(hyperClassif, samples, nHypotheses)
                                                             %, hypoThresh , normalize)

    if nargin < 3, nHypotheses= 1; end
    
    %if nargin < 4, hypoThresh= 0; end
    %if nargin < 5 || hypoThresh > 0, normalize= 1; end

    if nHypotheses > hyperClassif.nClasses, nHypotheses= hyperClassif.nClasses; end
    nSamples= length(samples);
    responses= zeros(nHypotheses, nSamples);
    decisionLevel= 1;
    responseValues= zeros(nHypotheses, nSamples);
    nHypos= zeros(1, nSamples);
    
    nLevels= hyperClassif.nLevels;
    nClasses= hyperClassif.nClasses;
    classIds= hyperClassif.classIds;
    filterClassifier= hyperClassif.filterClassifier;
    classifiers= hyperClassif.classifiers;
    featureParams= hyperClassif.featureParams;
    gCCs= hyperClassif.globalCenterCredits;
    cOs= hyperClassif.classOccurances;
    %cDs= hyperClassif.classDescriptions;
    cDRs= hyperClassif.centerDistanceRatios;
    
    classifConfig.keepHistory= 0;
    classifConfig.growthThresh= 0;
    classifConfig.scale= 10;
    classifConfig.growthFactor= 1.5;
    classifConfig.shrinkThresh= 0;
    classifConfig.vanishThresh= 0;
    classifConfig.scaleAdaptation= 'none';
    classifConfig.permuteSamples= 0;
    classifConfig.newClusterThresh= 0;
    classifConfig= createClassifierConfig_(classifConfig);
    
    %zeroWarns= 0;
    
    for sam= 1 : nSamples
        % generate base-segment info (transInfo, start, end)
        % ASSUMPTION: closed curves, with transInfo end-begin at 1st point
        
        sa= samples{sam};
        segments= sa.segments;
        [nSegLevels, nSegs]= size(segments);
%         if nSegLevels ~= nLevels, warning(['getHyperClassifierResponse: sample ' num2str(sam) ': segmentation levels do not match # classifers!']); end
        levelResults= cell(1, nLevels);
        votes= zeros(nLevels, nClasses);
        cenVotes= zeros(nLevels, nClasses);
        %responseFound= 0;
        if exist('classif', 'var'), clear classif; end
        segCount= 0;
        
        % compose and classify segments on all levels
        for level= min([nSegLevels nLevels]) : -1 : 1
            
            if strcmp(sa.config.segmentationMode, 'secondPower')
                feats= zeros(sum(featureParams(level).nFeatures)+5, nSegs);
                for seg= 1 : nSegs
                    if isempty(segments(level, seg).start) || segments(level, seg).start == 0, continue; end
                    segCount= segCount +1;
                    feats(6:end, seg)= getFeatures(sa, level, seg, featureParams(level).nFeatures, 25);
                    feats(1, segCount)= segments(level, seg).len;
                    feats(2, segCount)= segments(level, seg).startCoords(1);
                    feats(3, segCount)= segments(level, seg).startCoords(2);
                    feats(4, segCount)= segments(level, seg).endCoords(1);
                    feats(5, segCount)= segments(level, seg).endCoords(2);
                end
            elseif strcmp(sa.config.segmentationMode, 'allCombinations')
                lowLev= min([max([2^level - 2^(level-1) 1]) nSegLevels]);
                upLev= min([2^level + 2^(level-1) nSegLevels]) ;
                feats= zeros(sum(featureParams(level).nFeatures)+5, nSegs*(upLev-lowLev+1));
                segCount= 0;
                for lev= lowLev : upLev
                    for seg= 1 : nSegs
                        if isempty(segments(level, seg).start) || segments(level, seg).start == 0, continue; end
                        segCount= segCount +1;
                        feats(6:end, segCount)= getFeatures(sa, lev, seg, featureParams(level).nFeatures, 25);
                        feats(1, segCount)= segments(lev, seg).len;
                        feats(2, segCount)= segments(lev, seg).startCoords(1);
                        feats(3, segCount)= segments(lev, seg).startCoords(2);
                        feats(4, segCount)= segments(lev, seg).endCoords(1);
                        feats(5, segCount)= segments(lev, seg).endCoords(2);
                    end
                end
                feats(:, segCount:end)= [];
            end
            
            if level == 1
                [respondingClasses, responseVals]= getClassifierResponse(filterClassifier, feats(6:end, :));
            else
                [respondingClasses, responseVals]= getClassifierResponse(classifiers(level), feats(6:end, :));
            end
            
            levelResults{level}= [respondingClasses; responseVals];

            indices= levelResults{level}(1, :) > 0;
%             segClassProbs= zeros(nClasses, sum(indices));
            segClassProbs= zeros(max(classIds), sum(indices));
            if sum(indices) > 0
                lR1= levelResults{level}(1, indices);
                lR2= levelResults{level}(2, indices);
                nX= sum(gCCs(1, :, level));           
                nXs= gCCs(1, :, level) ./ nX;                     % p(xi)
                pXt= sum(nXs(lR1) .* lR2);           % p(Xt)
%                 for claz= 1 : nClasses
                for claz= 1 : max(classIds)
                    classIndex= find(classIds == claz);
                    if isempty(classIndex), continue; end
                    pClass= cOs(classIndex + 1) / cOs(1);       % p(claz)
                    nXCs= gCCs(classIndex + 1, :, level) ./ ...
                        sum(gCCs(classIndex + 1, :, level));      % p(xi | claz)
%                     segClassProbs(claz, :)= nXCs(lR1) .* lR2;    % p(xi | claz) * respVal
                    segClassProbs(classIndex, :)= nXCs(lR1) .* lR2;    % p(xi | claz) * respVal
                    pXtClass= sum(segClassProbs(claz, :));  % p(Xt | claz)
                    pClassXt= pXtClass * pClass / pXt;          % p(claz | Xt)
                    if isnan(pClassXt), pClassXt= 0; end
%                     votes(level, claz)= pClassXt;
                    votes(level, classIndex)= pClassXt;
                end
                votes(level, :)= votes(level, :) * 2^level;
            else
                votes(level, :)= 0;
            end
            
            if sum(indices) > 0
                segTypes= lR1;
                nSegTypes= size(lR1, 2);
                trainSamples= zeros(2, nSegTypes);
                trainIds= zeros(1, nSegTypes);
                weights= zeros(1, nSegTypes);
                count= 0;

                for seg= 1 : nSegTypes
                    segType= segTypes(seg);
                    len= feats(1, seg);
                    if segType == 0 || isempty(len) || len == 0, continue; end
                    a1= feats(2, seg);
                    b1= feats(3, seg);
                    a2= feats(4, seg);
                    b2= feats(5, seg);
                    a= a2 - a1;
                    b= b2 - b1;
                    aa= a^2; bb= b^2;
%                     for claz= 1 : nClasses
                    for claz= 1 : max(classIds)
%                         classIndex= classIds(claz);
                        classIndex= find(classIds == claz);
                        if isempty(classIndex), continue; end
                        r1= cDRs(classIndex, segType, 1) * len;
                        r2= cDRs(classIndex, segType, 2) * len;
                        rr1= r1^2; rr2= r2^2;
                        y1= 1/2*(aa*bb+b*(-aa*(-rr1+aa+bb-2*r2*r1-rr2)*(-rr1+aa+bb+2*r2*r1-rr2))^(1/2)+aa*rr1+a^4-aa*rr2)/(aa+bb)/a + a1;
                        y2= 1/2*(aa*b+rr1*b-rr2*b+b^3-(-aa*(-rr1+aa+bb-2*r2*r1-rr2)*(-rr1+aa+bb+2*r2*r1-rr2))^(1/2))/(aa+bb) + b1;
                        if ~isreal(y1) || ~isreal(y2) || isnan(y1) || isnan(y2), continue; end

                        count= count + 1;
                        trainSamples(1, count)= 1/2*(aa*bb-b*(-aa*(-rr1+aa+bb-2*r2*r1-rr2)*(-rr1+aa+bb+2*r2*r1-rr2))^(1/2)+aa*rr1+a^4-aa*rr2)/(aa+bb)/a + a1;
                        trainSamples(2, count)= 1/2*(aa*b+rr1*b-rr2*b+b^3+(-aa*(-rr1+aa+bb-2*r2*r1-rr2)*(-rr1+aa+bb+2*r2*r1-rr2))^(1/2))/(aa+bb) + b1;
                        trainIds(count)= classIndex;
                        weights(count)= segClassProbs(claz, seg);
                        count= count + 1;
                        trainSamples(1, count)= y1;
                        trainSamples(2, count)= y2;
                        trainIds(count)= classIndex;
                        weights(count)= segClassProbs(claz, seg);
                    end
                end

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                % evtl. �bertrieben, da segVotes nicht ausgewertet werden:
    %             if count == 0, continue; end

                if count > 0
                    if exist('classif', 'var')
                        classif= updateClassifier_(classif, trainSamples(:, 1:count), trainIds(1:count), weights);
                    else
                        classif= newClassifier_(trainSamples(:, 1:count), trainIds(1:count), weights, classifConfig);
                    end
                end
                if exist('classif', 'var')
                    [ss, ix]= sort(classif.correctResponses);
                    nLast= min([15 classif.nCenters-1]);
                    ids= classif.centerClassIds(ix(end-nLast:end));
                    hits= ss(end-nLast:end);
                    rads= classif.radii(end-nLast:end);
                    hits= hits ./ rads;
                    for claz= 1 : nClasses
                        cenVotes(level, claz)= sum(hits(ids==claz));
                    end
                    cenVotes(level, :)= cenVotes(level, :) * 2^level;
                end
            end
            
            
            
            
            totalVotes= sum([votes; cenVotes]);
            totalVotes= totalVotes ./ sum(totalVotes);
            [values, classInds]= sort(totalVotes, 'descend');
            if values(2) / values(1) < 0.7
                decisionLevel= level;
                break; 
            end
        end % level
        
    

        classInds(values <= 0)= 0;
        responses(1:nHypotheses, sam)= classInds(1:nHypotheses);
        responseValues(1:nHypotheses, sam)= values(1:nHypotheses);
    end % sam
%     zeroWarns
    
    
    
    
    
    