function config= createClassifierConfig(config)
% createClassifierConfig creates a struct 'config' with all parameters 
% needed to initialize and operate a classifier.
% - config: is either created from scratch - initialized with default
%   values, or it is completed with all missing fields. The later use is
%   intended for more advanced users who want to set some parameters to
%   other values than the default ones.

    if nargin < 1, config= []; end
    
    if ~isfield(config, 'type'), config.type= 'classifier'; end
    if ~isfield(config, 'logFilename'), config.logFilename= 'classifierLog.txt'; end
    if ~isfield(config, 'displayInfo'), config.displayInfo= 0; end
    if ~isfield(config, 'keepHistory'), config.keepHistory= 0; end
    
    if ~isfield(config, 'scale'), config.scale= -1; end
    if ~isfield(config, 'scalePercentage'), config.scalePercentage= 0.3; end
    if ~isfield(config, 'scaleAdaptation'), config.scaleAdaptation= 'neighbourMean'; end
        
    if ~isfield(config, 'permuteSamples'), config.permuteSamples= 1; end
    if ~isfield(config, 'reorganize'), config.reorganize= 0; end
    if ~isfield(config, 'nReorganizeSamples'), config.nReorganizeSamples= 300; end
    if ~isfield(config, 'reorgMaxSupport'), config.reorgMaxSupport= 1.5; end
    
    if ~isfield(config, 'radius'), config.radius= 1; end
    if ~isfield(config, 'growthFactor'), config.growthFactor= 1.1; end
    if ~isfield(config, 'newClusterThresh'), config.newClusterThresh= 0.55; end
    if ~isfield(config, 'growthThresh'), config.growthThresh= 0.95; end
    if ~isfield(config, 'shrinkThresh'), config.shrinkThresh= 0.75; end
    if ~isfield(config, 'vanishThresh'), config.vanishThresh= 0.3; end
    
    if ~isfield(config, 'generateNewClasses'), config.generateNewClasses= 0; end
    if ~isfield(config, 'recycleRubbish'), config.recycleRubbish= 0; end
    
    if ~isfield(config, 'nearestNeighbour'), config.nearestNeighbour= 0; end