function [respondingClasses, responseValues, classResponses]= getClassifierResponseAnalytic(classifier, samples, supportedClasses, supportFactors)
% getClassifierResponseAnalytic classifies the provided samples using the 
% specified classifier.
% - classifier: trained classifier
% - samples: nDims x nSamples-matrix of data vectors to be classified.
% - supportedClasses, supportFactors: (optional) two vectors of the same
%   length naming class-ids and corresponding factors. All cluster-radii of the
%   specified classes will be multiplied by the corresponding factors.
%   Thus, individual classes can be enforced to stronger/weaker responses.
% - respondingClasses: 1 x nSamples vector holding the class-ids that were
%   assigned to the corresponding samples (, because of having the highest 
%   response-value).
% - responseValues: the response-values of the highest responding classes.
% - classResponses: nClasses x nSamples-matrix holding response-values of
%   all classes for each sample-vector.

    radii= classifier.radii.^2;
    if nargin == 4
        for i= 1 : length(supportedClasses)
            indices= classifier.centerClassIds == supportedClasses(i);
            radii(indices)= radii(indices) .* supportFactors(i).^2;
        end
    end
        
    nSamples= size(samples, 2);
%     respondingClasses= zeros(1, nSamples);
%     responseValues= zeros(1, nSamples);
    centers= classifier.centers;
    credits= classifier.credits;
     
%     if classifier.config.displayInfo == 1, fprintf('computing responses for each center \n'); end
    responses= zeros(classifier.nCenters, nSamples);
    for cen= 1 : classifier.nCenters
        centerMatrix= centers(:, ones(1, nSamples).*cen);
        diffVecs= centerMatrix - samples;
        distSquared= sum(diffVecs.^2);
        responses(cen, :)= (1 - distSquared ./ radii(cen)) * credits(cen);
    end
    if classifier.config.nearestNeighbour == 1
        minVal= min(min(responses));
        if minVal < 0, responses= responses + abs(minVal); end
    end
    responses(responses < 0)= 0;
    classResponses= zeros(classifier.nClasses, nSamples);
    for claz= 1 : classifier.nClasses
        cis= classifier.centerClassIds == classifier.classIds(claz);
        if sum(cis) == 1
            classResponses(claz, :)= responses(cis, :);
        else
            classResponses(claz, :)= sum(responses(cis, :));
        end
    end
    [responseValues, maxClasses]= max(classResponses);
    if classifier.nClasses == 1
        respondingClasses= classifier.classIds(1);
    else
        respondingClasses= classifier.classIds(maxClasses);
    end
%     respondingClasses(responseValues == 0)= -1;
    respondingClasses(responseValues == 0)= 0;