function classifier= reorganizeClassifier_(classifier, classIds)

    if nargin < 2, classIds= classifier.classIds; end
    
    centers= classifier.centers;
    radii= classifier.radii;
    centerClassIds= classifier.centerClassIds;
    nCenters= classifier.nCenters;
    correctResponses= classifier.correctResponses;
    totalResponses= classifier.totalResponses;
    credits= classifier.credits;
    growthThresh= classifier.config.growthThresh;
    
    maxSup= classifier.config.reorgMaxSupport; 
    minSup= classifier.config.growthFactor; 
    maxCred= 1; 
    minCred= growthThresh;
    supports= (maxSup - minSup) / (maxCred - minCred) * (credits - minCred) + minSup;
    supportedRadii= radii .* supports;
    
    nMerges= 0;
    
    for claz= 1 : length(classIds)
        centerIds= 1 : nCenters;
        ctbm= centerClassIds == classIds(claz); % clusters to be merged
        fc= ctbm == 0;                          % foreign clusters
        fc= centerIds(fc);
        nFc= length(fc);
        ctbm= logical(ctbm .* (credits >= growthThresh));
        ctbm= centerIds(ctbm);
        nCtbm= length(ctbm);
        cen= 1;
        while cen <= nCtbm
            % distances to member clusters
            c(1)= ctbm(cen);
            centerMatrix= centers(:, ones(1, nCtbm-cen).*c(1));
            diffVecs= centers(:, ctbm(cen+1 : end)) - centerMatrix;
            dists= sqrt(sum(diffVecs.^2));
            supportedDists= dists - supportedRadii(ctbm(cen+1 : end)) - supportedRadii(c(1));
            [dist, ind]= min(supportedDists);
            if dist < 0
                % merge with nearest member cluster (if close enough)
                c(2)= ctbm(cen + ind);
                [minRad, minRadI]= min([radii(c(1)) radii(c(2))]);
                [maxRad, maxRadI]= max([radii(c(1)) radii(c(2))]);
                if minRad + dist < maxRad
                    newRadius= maxRad;
                    newCenter= centers(:, c(maxRadI));
                else
                    newRadius= (minRad + maxRad + dist) / 2;
                    if minRadI ~= 1, diffVecs(:, ind)= -diffVecs(:, ind); end
                    newCenter= centers(:, c(minRadI)) + (newRadius - radii(c(minRadI))) * diffVecs(:, ind)/dist;
                end
                newCResps= sum(correctResponses(c));
                newTResps= sum(totalResponses(c));
                % check new cluster w.r.t. foreign clusters
                centerMatrix= newCenter(:, ones(1, nFc));
                diffVecs= centers(:, fc) - centerMatrix;
                dists= sqrt(sum(diffVecs.^2));
                overlap= dists - newRadius - radii(fc);
                inds= overlap < 0;
                rfc= fc(inds); %responding foreign clusters
                facts= abs(overlap(inds)) ./ (radii(rfc).*2);
                newTResps= sum([newTResps facts .* correctResponses(rfc)]);
                newCred= newCResps / newTResps;
                if newCred >= min(credits(c))
                    nMerges= nMerges + 1;
                    % set second merged cluster to new one
                    centers(:, c(2))= newCenter;
                    radii(c(2))= newRadius;
                    correctResponses(c(2))= newCResps;
                    totalResponses(c(2))= newTResps;
                    credits(c(2))= newCred;
                    % delete first cluster from all lists
                    centers(:, c(1))= [];
                    radii(c(1))= [];
                    correctResponses(c(1))= [];
                    totalResponses(c(1))= [];
                    credits(c(1))= [];
                    centerClassIds(c(1))= [];
                    nCenters= nCenters - 1;
                    supportedRadii(c(1))= [];
                    % update all indices that stay in use
                    cen= cen - 1;
                    nCtbm= nCtbm - 1;
                    ctbm(1)= [];
                    ctbm= ctbm - 1;
                    inds= fc > c(1);
                    fc(inds)= fc(inds) - 1;
                end
            end
            cen= cen + 1;
        end
    end
    
    classifier.nCenters= nCenters;
    classifier.centers= centers;
    classifier.radii= radii;
    classifier.centerClassIds= centerClassIds;
    classifier.correctResponses= correctResponses;
    classifier.totalResponses= totalResponses;
    classifier.credits= credits;
    classifier.nSinceLastReorg= 0;
    
    
    if classifier.config.displayInfo == 1, fprintf('reorganizeClassifier: merged %d clusters\n', nMerges); end