function classifier= updateClassifier_(classifier, samples, classIds, weights)

    if classifier.config.displayInfo == 1, fprintf('updateClassifier started...\n'); end
    % initialize parameters
    nSamples= size(samples, 2);
    if nargin < 4 || isempty(weights), weights= ones(1, nSamples); end
    
    estimateScaleAgain= 0;
    nNewClasses= 0;
    
    % get nClasses and list of ids
    minId= min(classIds);
    maxId= max(classIds);
    for id= minId : maxId
        if any(classIds == id) && all(classifier.classIds ~= id)
            classifier.nClasses= classifier.nClasses + 1;
            classifier.classIds(classifier.nClasses)= id;
            estimateScaleAgain= 1;
            nNewClasses= nNewClasses + 1;
        end
    end
    
    nSamplesOld= classifier.nSamples;
    classifier.nSamples= classifier.nSamples + nSamples;
    if classifier.config.displayInfo == 1
        fprintf('samples to be trained: %d\n', nSamples);
        fprintf('updateClassifier: new classes initialized: %d\n', nNewClasses);
    end
    
    centers= classifier.centers;
    radii= classifier.radii;
    centerClassIds= classifier.centerClassIds;
    nCenters= classifier.nCenters;
    correctResponses= classifier.correctResponses;
    totalResponses= classifier.totalResponses;
    credits= classifier.credits;
    centerIds= 1 : nCenters;
    nSinceLastReorg= classifier.nSinceLastReorg;
    nReorganizeSamples= classifier.config.nReorganizeSamples;
    reorganize= classifier.config.reorganize;
    growthThresh= classifier.config.growthThresh;
    growthFactor= classifier.config.growthFactor;
    shrinkThresh= classifier.config.shrinkThresh;
    vanishThresh= classifier.config.vanishThresh;
    newClusterThresh= classifier.config.newClusterThresh;
    scale= classifier.config.scale;
    keepHistory= classifier.config.keepHistory;
        
    if keepHistory == 1
        dummy= max([100 nCenters]);
        nCenterHist= zeros(1, classifier.nSamples);
        nCenterHist(1 : nSamplesOld)= classifier.history.nCenters;
        radiiHist= zeros(classifier.nSamples, dummy);
        radiiHist(1 : nSamplesOld, 1:size(classifier.history.radii, 2))= classifier.history.radii;
        creditHist= zeros(classifier.nSamples, dummy);
        creditHist(1 : nSamplesOld, 1:size(classifier.history.credits, 2))= classifier.history.credits;
        nRespHist= zeros(classifier.nSamples, dummy);
        nRespHist(1 : nSamplesOld, 1:size(classifier.history.totalResponses, 2))= classifier.history.totalResponses;
        ccIdHist= zeros(classifier.nSamples, dummy);
        ccIdHist(1 : nSamplesOld, 1:size(classifier.history.centerClassIds, 2))= classifier.history.centerClassIds;
        centerHist= zeros(classifier.nDims, dummy, classifier.nSamples);
        centerHist(1 : classifier.nDims, 1 : size(classifier.history.centers, 2), 1 : nSamplesOld)= classifier.history.centers;
%         centerDiffHist= zeros(nDims, dummy, classifier.nSamples);
        
%         nCenterHist= classifier.history.nCenters;
%         centerHist= classifier.history.centers;
% %         centerDiffHist= classifier.history.centerDiffs;
%         radiiHist= classifier.history.radii;
%         creditHist= classifier.history.credits;
%         nRespHist= classifier.history.totalResponses;
%         ccIdHist= classifier.history.centerClassIds;
    end
      
    % estimate scale, i.e. distance between samples
    if scale == -1 && estimateScaleAgain == 1
        estimSamples= samples;
        fact= (classifier.nSamples - nSamples) / sum(correctResponses);
        for cen= 1 : nCenters
            estimSamples= [estimSamples centers(:, ones(1, ceil(correctResponses(cen)*fact)) .* cen)];
        end
        nEstimSamples= size(estimSamples, 2);
        indices= randperm(nEstimSamples);
        estimSamples= estimSamples(:, indices);
        a= min([200 nEstimSamples]);
        dd= pdist(estimSamples(:, 1:a)');
        dd(dd == 0)= [];
        edges= min(dd) : (max(dd)-min(dd))/sqrt(length(dd)) : max(dd);
        n= histc(dd, edges);
% 	    figure;bar(edges, n);
        mass= sum(n);
        for i= 1 : length(n)
            if sum(n(1:i))/mass > classifier.config.scalePercentage, break; end
        end
        if i == 1, i= 2; end
        scale= edges(i);
        classifier.config.scale= scale;
    end
        
    %% permute samples + ids
    if classifier.config.permuteSamples
        indices= randperm(nSamples);
        samples= samples(:, indices);
        classIds= classIds(indices);
        weights= weights(indices);
    end
   
    for sam= 1 : nSamples
%         if mod(sam, 100) == 0
%             fprintf('.'); 
%             if mod(sam, 1000) == 0, fprintf('\n'); end 
%         end
        % calculate responses for all clusters
        sampleMatrix= samples(:, ones(1, nCenters).*sam);
        diffVecs= centers - sampleMatrix;
        distSquared= sum(diffVecs.^2);
        responses= 1 - distSquared ./ (radii.^2);
%         responses= 1 - sum((centers - sampleMatrix).^2) ./ (radii).^2;

        % compute indices
        cis= centerClassIds == classIds(sam);  % indices of correct clusters
        fis= cis == 0;  % indices of foreign clusters
        ris= responses >= 0; % indices of responding clusters
        nris= ris == 0; % indices of not responding clusters
        cris= logical(cis .* ris); % correctly responding cluster indices
        cnris= logical(cis .* nris); % indices of correct clusters that have not responded
        fris= logical(fis .* ris); % falsely responding cluster indices
        
        % update falsely responding clusters
        totalResponses(fris)= totalResponses(fris) + weights(sam);
        credits(fris)= correctResponses(fris) ./ totalResponses(fris);
        
        % update correctly responding clusters
        correctResponses(cris)= correctResponses(cris) + weights(sam);
        totalResponses(cris)= totalResponses(cris) + weights(sam);
        credits(cris)= correctResponses(cris) ./ totalResponses(cris);
        
        % update correct but not responding clusters
        cnris(credits < growthThresh)= 0; % that may be grown
        responses_= 1 - distSquared(cnris) ./ (radii(cnris) .* ((growthFactor-1).*weights(sam)+1)).^2;
        ctbg= centerIds(cnris);
        ctbg(responses_ < 0)= []; % indices of clusters to be grown
        if sum(ctbg) > 0
            dist= sqrt(distSquared(ctbg));
            offset= (dist - radii(ctbg)) ./ 2;
            centers(ctbg)= centers(ctbg) + offset .* diffVecs(ctbg) ./ dist;
            radii(ctbg)= radii(ctbg) + offset;
            correctResponses(ctbg)= correctResponses(ctbg) + weights(sam);
            totalResponses(ctbg)= totalResponses(ctbg) + weights(sam);
            credits(ctbg)= correctResponses(ctbg) ./ totalResponses(ctbg);
        end
        
        % calculate responses for all clusters
        % - not necessary, as responses stay the same (ctbg are 0)
        
        % sum responses of member clusters
        support= sum(responses(cris) .* credits(cris));
        
        % if necessary initialize new cluster
        if support <= newClusterThresh
            nCenters= nCenters + 1;
            centers(:, nCenters)= samples(:, sam);
            centerClassIds(nCenters)= classIds(sam);
            correctResponses(nCenters)= weights(sam);
            totalResponses(nCenters)= weights(sam);
            credits(nCenters)= 1;
            centerIds= 1 : nCenters;
            % scale adaptation
            if strcmp(classifier.config.scaleAdaptation, 'none')
                radii(nCenters)= scale;
            elseif strcmp(classifier.config.scaleAdaptation, 'allMean')
                scale= mean(radii);
                radii(nCenters)= scale;
            elseif strcmp(classifier.config.scaleAdaptation, 'neighbourMean')
                allMean= mean(radii);
                neighbourIndices= sqrt(distSquared) <= (2 * allMean);
                if sum(neighbourIndices) > 2
                    radii(nCenters)= mean(radii(neighbourIndices));
                else
                    radii(nCenters)= allMean;
                end
            end
        end
        
        % shrink/delete cluster
        if sum(fris) > 0
            sis= credits < shrinkThresh; % indices of clusters below shrinkThresh
            sis= logical(sis(1:length(fris)) .* fris);
            radii(sis)= radii(sis) ./ growthFactor;
            
            dis= credits < vanishThresh; % indices of clusters below vanishThresh
            if sum(dis) > 0
                dis= logical(dis(1:length(fris)) .* fris);
                centers(:, dis)= [];
                radii(dis)= [];
                centerClassIds(dis)= [];
                correctResponses(dis)= [];
                totalResponses(dis)= [];
                credits(dis)= [];
                nCenters= nCenters - sum(dis);
            end
        end
        % reorganize Classifier if necessary
        nSinceLastReorg= nSinceLastReorg + 1;
        if reorganize == 1 && nSinceLastReorg >= nReorganizeSamples
            classifier.nCenters= nCenters;
            classifier.centers= centers;
            classifier.radii= radii;
            classifier.centerClassIds= centerClassIds;
            classifier.correctResponses= correctResponses;
            classifier.totalResponses= totalResponses;
            classifier.credits= credits;
            classifier.nSamples= nSamples;
            
            classifier= reorganizeClassifier(classifier);
            
            centers= classifier.centers;
            radii= classifier.radii;
            centerClassIds= classifier.centerClassIds;
            nCenters= classifier.nCenters;
            correctResponses= classifier.correctResponses;
            totalResponses= classifier.totalResponses;
            credits= classifier.credits;
            centerIds= 1 : nCenters;
            nSinceLastReorg= classifier.nSinceLastReorg;
        end
        if sum(isnan(radii)) > 0, error('updateClassifier: radii are NaN!!'); end
        if keepHistory == 1
            nCenterHist(sam)= nCenters;
            radiiHist(sam, 1:length(radii))= radii;
            creditHist(sam, 1:length(credits))= credits;
            nRespHist(sam, 1:length(totalResponses))= totalResponses;
            ccIdHist(sam, 1:length(centerClassIds))= centerClassIds;
            centerHist(1:size(centers, 1), 1:size(centers, 2), sam)= centers;
        end
    end
    
    classifier.nCenters= nCenters;
    classifier.centers= centers;
    classifier.radii= radii;
    classifier.centerClassIds= centerClassIds;
    classifier.correctResponses= correctResponses;
    classifier.totalResponses= totalResponses;
    classifier.credits= credits;
    classifier.nSinceLastReorg= nSinceLastReorg;
    
    
    if keepHistory == 1
        classifier.history.nCenters= nCenterHist;
        classifier.history.centers= centerHist;
%         classifier.history.centerDiffs= centerDiffHist;
        classifier.history.radii= radiiHist;
        classifier.history.credits= creditHist;
        classifier.history.totalResponses= nRespHist;
        classifier.history.centerClassIds= ccIdHist;
    end